@@ -340,7 +340,7 @@ char *amap_strdup(char *string) {
 
 
 // AMAP_MEMDUP //
-char *amap_memdup(char *string, int len) {
+char *amap_memdup(unsigned char *string, int len) {
   char *ptr;
 
   if (string == NULL)
@@ -412,7 +412,7 @@ amap_struct_triggers *read_file_triggers(char *extension, char *filename, amap_s
   f = amap_open_file(filename, "trigger", extension, opt->verbose);
   if (opt->verbose > 1)
     printf("\n");
-  if (((amap_struct_triggers*) triggers = (amap_struct_triggers*) trigger = (amap_struct_triggers*) malloc(sizeof(amap_struct_triggers))) == NULL)
+  if ((triggers = trigger = (amap_struct_triggers*) malloc(sizeof(amap_struct_triggers))) == NULL)
     amap_error("malloc failed");
   memset(trigger, 0, sizeof(amap_struct_triggers));
 
@@ -587,7 +587,7 @@ amap_struct_responses *read_file_responses(char *extension, char *filename, amap
   f = amap_open_file(filename, "response", extension, opt->verbose);
   if (opt->verbose > 1)
     printf("\n");
-  if (((amap_struct_responses*) responses = (amap_struct_responses*) response = (amap_struct_responses*) malloc(sizeof(amap_struct_responses))) == NULL)
+  if ((responses = response = (amap_struct_responses*) malloc(sizeof(amap_struct_responses))) == NULL)
     amap_error("malloc failed");
   memset(response, 0, sizeof(amap_struct_responses));
 
@@ -632,7 +632,7 @@ amap_struct_responses *read_file_responses(char *extension, char *filename, amap
         for (i = 0; i < strlen(triggerptr); i++)
           if (triggerptr[i] == ',')
             count++;
-        if (((amap_struct_triggerptr*) triggerptr_tmp = response->triggerptr = malloc(sizeof(amap_struct_triggerptr))) == NULL)
+        if ((triggerptr_tmp = response->triggerptr = malloc(sizeof(amap_struct_triggerptr))) == NULL)
           amap_error("malloc failed");
         triggerptr_tmp->next = NULL;
         
@@ -744,7 +744,7 @@ amap_struct_targets *read_file_nmap(char *filename, amap_struct_options *opt) {
   int count = 0;
 
   f = amap_open_file(filename, "nmap", "", opt->verbose);
-  if (((amap_struct_targets*) targets = (amap_struct_targets*) target = (amap_struct_targets*) malloc(sizeof(amap_struct_targets))) == NULL)
+  if ((targets = target = (amap_struct_targets*) malloc(sizeof(amap_struct_targets))) == NULL)
     amap_error("malloc failed");
   memset(target, 0, sizeof(amap_struct_targets));
   if ((port_tmp = target->ports = (amap_struct_ports *) malloc(sizeof(amap_struct_ports))) == NULL)
@@ -870,7 +870,7 @@ char *amap_build_time(char *today, int len) {
 
 
 // AMAP_PRINTABLE_BANNER_STRING //
-char *amap_printable_banner_string(char *string, int slen, char *banner, int blen) {
+char *amap_printable_banner_string(unsigned char *string, int slen, char *banner, int blen) {
   int i = 0;
   int j = 0;
 
@@ -1060,7 +1060,10 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
     if (coms[i].active == AMAP_CONNECT_ACTIVE && coms[i].socket != -1) {
       if (coms[i].ssl_enabled) {
 #ifdef OPENSSL
-        len = SSL_read(coms[i].ssl_socket, buf, sizeof(buf));
+        if (SSL_pending(coms[i].ssl_socket) > 0)
+          len = SSL_read(coms[i].ssl_socket, buf, sizeof(buf));
+        else
+          len = 0;
 #endif
       } else {
         len = recv(coms[i].socket, buf, sizeof(buf), 0);
@@ -1078,14 +1081,14 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
       }
       // oh yeah, we received data!
       if (len > 0 && opt->banner_only) {
-        printf("Banner on %s:%d/%s : %s\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_printable_banner_string(buf, len, banner, sizeof(banner)));
+        printf("Banner on %s:%d/%s : %s\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_printable_banner_string((char *)buf, len, banner, sizeof(banner)));
         if (opt->logfile != NULL) {
           if (opt->machine_readable) {
-            fprintf(opt->logfile, "%s:%d:%s:%s::%s:%s:", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), AMAP_UFO, amap_printable_banner_string(buf, len, banner, sizeof(banner)));
+            fprintf(opt->logfile, "%s:%d:%s:%s::%s:%s:", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), AMAP_UFO, amap_printable_banner_string((char *)buf, len, banner, sizeof(banner)));
             amap_banner_string(opt->logfile, buf, len);
             fprintf(opt->logfile, "\n");
           } else
-            fprintf(opt->logfile, "Banner on %s:%d/%s : %s\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_printable_banner_string(buf, len, banner, sizeof(banner)));
+            fprintf(opt->logfile, "Banner on %s:%d/%s : %s\n", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_printable_banner_string((char *)buf, len, banner, sizeof(banner)));
         }
         coms[i].port->skip = 1;
 #ifdef OPENSSL
@@ -1116,7 +1119,7 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
             if ((opt->one_is_enough == 0 || found == 0) && amap_lookup_id(coms[i].port->ids, "echo") == 0) {
               if (len >= response->min_length && len <= response->max_length && (response->ip_prot == AMAP_PROTO_BOTH || response->ip_prot == coms[i].port->ip_prot)
                   && amap_lookup_id(coms[i].port->ids, response->id) == 0 && (response->triggerptr == NULL || amap_lookup_triggerptr(response->triggerptr, coms[i].trigger->id) == 1)) {
-                if (pcre_exec(response->pattern, response->hints, buf, len, 0, 0, offsets, sizeof(offsets)) >= 0) {
+                if (pcre_exec(response->pattern, response->hints, (char *)buf, len, 0, 0, offsets, sizeof(offsets)) >= 0) {
                   found++;
                   amap_add_id(coms[i].port, response->id);
                   if (strcmp(response->id, "ssl") == 0 || strncmp(response->id, "ssl-", 4) == 0)
@@ -1129,13 +1132,13 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
                     info[0] = 0;
                   if (opt->logfile != NULL) {
                     if (opt->machine_readable) {
-                      fprintf(opt->logfile, "%s:%d:%s:%s:%s:%s:%s:", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), coms[i].ssl_enabled ? "SSL" : "", response->id, amap_printable_banner_string(coms[i].response, coms[i].response_length, banner, sizeof(banner)));
+                      fprintf(opt->logfile, "%s:%d:%s:%s:%s:%s:%s:", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), coms[i].ssl_enabled ? "SSL" : "", response->id, amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
                       amap_banner_string(opt->logfile, coms[i].response, coms[i].response_length);
                       fprintf(opt->logfile, "\n");
                     } else {
                       fprintf(opt->logfile, "Protocol on %s:%d/%s%s%smatches %s", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info, response->id);
                       if (opt->banner) 
-                        fprintf(opt->logfile, " - banner: %s\n", amap_printable_banner_string(coms[i].response, coms[i].response_length, banner, sizeof(banner)));
+                        fprintf(opt->logfile, " - banner: %s\n", amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
                       else
                         fprintf(opt->logfile, "\n");
                       if (opt->dump_all) {
@@ -1147,7 +1150,7 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
                   }
                   printf("Protocol on %s:%d/%s%s%smatches %s", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].ssl_enabled ? " over SSL " : " ", info, response->id);
                   if (opt->banner) 
-                    printf(" - banner: %s\n", amap_printable_banner_string(coms[i].response, coms[i].response_length, banner, sizeof(banner)));
+                    printf(" - banner: %s\n", amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
                   else
                     printf("\n");
                   if (opt->dump_all) {
@@ -1179,7 +1182,7 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
               printf("Please send this output and the name of the application to %s:\n", AMAP_EMAIL);
               amap_dump_string(stdout, coms[i].response, coms[i].response_length, 16);
             }
-            coms[i].port->unknown_response = amap_memdup(coms[i].response, coms[i].response_length);
+            coms[i].port->unknown_response = (unsigned char *)amap_memdup((char *)coms[i].response, coms[i].response_length);
             coms[i].port->unknown_response_length = coms[i].response_length;
           }
           // now shutdown the connection
@@ -1202,7 +1205,7 @@ void amap_read_responses(amap_struct_coms *coms, amap_struct_responses *response
             amap_add_id(coms[i].port, banner);
             if (opt->logfile != NULL) {
               if (opt->machine_readable) {
-                fprintf(opt->logfile, "%s:%d:%s:%s:%s:%s:%s:", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), coms[i].ssl_enabled ? "SSL" : "", coms[i].port->ids->id, amap_printable_banner_string(coms[i].response, coms[i].response_length, banner, sizeof(banner)));
+                fprintf(opt->logfile, "%s:%d:%s:%s:%s:%s:%s:", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(coms[i].port->skip), coms[i].ssl_enabled ? "SSL" : "", coms[i].port->ids->id, amap_printable_banner_string((char *)coms[i].response, coms[i].response_length, banner, sizeof(banner)));
                 amap_banner_string(opt->logfile, coms[i].response, coms[i].response_length);
                 fprintf(opt->logfile, "\n");
               } else
@@ -1277,7 +1280,8 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
   int ii;
   int res;
   int error;
-  int error_len = sizeof(error);
+  socklen_t error_len = sizeof(error);
+  socklen_t sock_len = sizeof(struct sockaddr);
   int maxfd = -1;
   char *ptr;
   long int *j;
@@ -1310,7 +1314,7 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
       if (FD_ISSET(coms[i].socket, &rfd) || FD_ISSET(coms[i].socket, &wfd)) {
       // somethings here ...
         error = 0;
-        if (getsockopt(coms[i].socket, SOL_SOCKET, SO_ERROR, &error, &error_len) < 0 || error != 0) {
+        if (getsockopt(coms[i].socket, SOL_SOCKET, SO_ERROR, &error, (socklen_t *)&error_len) < 0 || error != 0) {
           if (coms[i].port->skip < 1 && (opt->portscanner == 0 || opt->verbose) && opt->quiet == 0)
             amap_warn("Could not connect (unreachable) to %s:%d/%s, disabling port (EUNKN)", coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           coms[i].port->skip = 2;
@@ -1366,14 +1370,14 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
           if (opt->ipv6) {
             struct sockaddr_in6 sa;
             int t = sizeof(sa);
-            getsockname(coms[i].socket, (struct sockaddr*)&sa, &t);
+            getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
             printf("DEBUG: socket %d/%d to %s:%d/%s is now RETRY CONNECT\n", coms[i].socket, htons(sa.sin6_port), coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           } else
 #endif
           {
             struct sockaddr_in sa;
             int t = sizeof(sa);
-            getsockname(coms[i].socket, (struct sockaddr*)&sa, &t);
+            getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
             printf("DEBUG: socket %d/%d to %s:%d/%s is now RETRY CONNECT\n", coms[i].socket, htons(sa.sin_port), coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           }
         }
@@ -1428,18 +1432,18 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
           if (opt->ipv6) {
             struct sockaddr_in6 sa;
             int t = sizeof(sa);
-            getpeername(coms[i].socket, (struct sockaddr*)&sa, &t);
+            getpeername(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
             dport = htons(sa.sin6_port);
-            getsockname(coms[i].socket, (struct sockaddr*)&sa, &t);
+            getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
             printf("DEBUG: socket %d/%d->%d to %s:%d/%s became RETRY CONNECT\n", coms[i].socket, htons(sa.sin6_port), dport, coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           } else
 #endif
           {
             struct sockaddr_in sa;
             int t = sizeof(sa);
-            getpeername(coms[i].socket, (struct sockaddr*)&sa, &t);
+            getpeername(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
             dport = htons(sa.sin_port);
-            getsockname(coms[i].socket, (struct sockaddr*)&sa, &t);
+            getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
             printf("DEBUG: socket %d/%d->%d to %s:%d/%s became RETRY CONNECT\n", coms[i].socket, htons(sa.sin_port), dport, coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           }
         }
@@ -1548,18 +1552,18 @@ void amap_check_connects(amap_struct_coms *coms, amap_struct_scaninfo *scaninfo,
             if (opt->ipv6) {
               struct sockaddr_in6 sa;
               int t = sizeof(sa);
-              getpeername(coms[i].socket, (struct sockaddr*)&sa, &t);
+              getpeername(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
               dport = htons(sa.sin6_port);
-              getsockname(coms[i].socket, (struct sockaddr*)&sa, &t);
+              getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
               printf("DEBUG: socket %d/%d->%d to %s:%d/%s became ACTIVE (trigger %s send)\n", coms[i].socket, ntohs(sa.sin6_port), dport, coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].trigger->id);
             } else
 #endif
             {
               struct sockaddr_in sa;
               int t = sizeof(sa);
-              getpeername(coms[i].socket, (struct sockaddr*)&sa, &t);
+              getpeername(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
               dport = htons(sa.sin_port);
-              getsockname(coms[i].socket, (struct sockaddr*)&sa, &t);
+              getsockname(coms[i].socket, (struct sockaddr*)&sa, (socklen_t *)&t);
               printf("DEBUG: socket %d/%d->%d to %s:%d/%s became ACTIVE (trigger %s send)\n", coms[i].socket, ntohs(sa.sin_port), dport, coms[i].target->target, coms[i].port->port, coms[i].port->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", coms[i].trigger->id);
             }
           }
@@ -1812,7 +1816,7 @@ int amap_scan(int scanmode, amap_struct_targets *targets, amap_struct_triggers *
                   fcntl(s, F_SETFL, O_NONBLOCK);
                   coms[i].timer = time(NULL);
                   coms[i].socket = s;
-                  /*(char*)*/ coms[i].sockaddr = (struct sockaddr*) amap_memdup((char *) glob_sockaddr, glob_sockaddr_len);
+                  /*(char*)*/ coms[i].sockaddr = (struct sockaddr*) amap_memdup((unsigned char *) glob_sockaddr, glob_sockaddr_len);
                   coms[i].sockaddr_len = glob_sockaddr_len;
                   scaninfo.running++;
                   errno = 0;
@@ -2087,7 +2091,7 @@ int amap_main(amap_struct_options *opt, int argc, char *argv[]) {
       responses_tmp = (amap_struct_responses*) responses_tmp->next;
     }
   } else {
-    if (((amap_struct_triggers*) triggers = (amap_struct_triggers*) malloc(sizeof(amap_struct_triggers))) == NULL)
+    if (( triggers = (amap_struct_triggers*) malloc(sizeof(amap_struct_triggers))) == NULL)
       amap_error("malloc failed");
     triggers->next = NULL;
     triggers->id = strdup("NULL");
@@ -2288,7 +2292,7 @@ int amap_main(amap_struct_options *opt, int argc, char *argv[]) {
           printf(" %s:%d/%s", target_tmp->target, port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp");
           if (opt->logfile != NULL) {
             if (opt->machine_readable) {
-              fprintf(opt->logfile, "%s:%d:%s:%s:%s:%s:%s:", target_tmp->target, port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(port_tmp->skip), port_tmp->ssl ? "SSL" : "", AMAP_UFO, amap_printable_banner_string(port_tmp->unknown_response, port_tmp->unknown_response_length, banner, sizeof(banner)));
+              fprintf(opt->logfile, "%s:%d:%s:%s:%s:%s:%s:", target_tmp->target, port_tmp->port, port_tmp->ip_prot == AMAP_PROTO_TCP ? "tcp" : "udp", amap_skip_translate(port_tmp->skip), port_tmp->ssl ? "SSL" : "", AMAP_UFO, amap_printable_banner_string((char *)port_tmp->unknown_response, port_tmp->unknown_response_length, banner, sizeof(banner)));
               amap_banner_string(opt->logfile, port_tmp->unknown_response, port_tmp->unknown_response_length);
               fprintf(opt->logfile, "\n");
             } else
