@@ -1,6 +1,6 @@
 #ifndef _AMAP_H
 
-/* AMAP - Application MAPper Copyright (c) 2003 van Hauser and DJ RevMoon
+/* AMAP - Application MAPper Copyright (c) 2003-2005 van Hauser and DJ RevMoon
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 #define AMAP_PROGRAM   "amap"
-#define AMAP_VERSION   "5.1"
+#define AMAP_VERSION   "5.2"
 #define AMAP_YEAR      "2005"
 #define AMAP_AUTHOR    "van Hauser"
 #define AMAP_EMAIL     "amap-dev@thc.org"
@@ -146,7 +146,7 @@ typedef struct {
   char rpc;
   char skip;
   int unknown_response_length;
-  unsigned char *unknown_response;
+  char *unknown_response;
   amap_struct_identifications *ids;
   struct amap_struct_ports *next;
 } amap_struct_ports;
