@@ -73,7 +73,7 @@ int main(int argc, char *argv[])
   int dont_exit_when_response = 0;
   long int connect_delay = 1;
   long int close_delay = 250;
-  unsigned char *str = NULL, *str_old;
+  char *str = NULL, *str_old;
   int str_len = 0, str_len_old = 0;
   int i, j;
   int s;
@@ -103,14 +103,14 @@ int main(int argc, char *argv[])
   while ((i = getopt(argc, argv, "M:N:SVem:n:p:uvw:")) >= 0) {
     switch (i) {
     case 'M':
-      if ((str = index(optarg, ',')) == NULL) {
+      if ((str = (unsigned char *)index(optarg, ',')) == NULL) {
 	fprintf(stderr,
 		"Error: Syntax is \"-M min,max\", e.g. \"-M 1,256\".\n");
 	exit(-1);
       }
       *str++ = 0;
       send_min = atoi(optarg);
-      send_max = atoi(str);
+      send_max = atoi((char *)str);
       if (send_min < 3 || send_max < 3 || send_min > 1024 || send_max > 1024
 	  || send_min > send_max) {
 	fprintf(stderr,
